/**
 * Interface specifying where a remote chunk is hosted.
 * It represents the output of {@link RemoteChunkResolver} function used by {@link ChunkManager}.
 */
export interface RemoteChunkLocation {
    /**
     * A URL to remote location, where to download a chunk from.
     *
     * Example: for `chunkId: 'TeacherModule'` the `url` can look like this:
     * `https://myapp.com/assets/TeacherModule`.
     */
    url: string;
    /**
     * Whether not to add chunk's default extension by default. If your chunk has different
     * extension than `.chunk.bundle` you should set this flag to `true` and add extension to the `url`.
     */
    excludeExtension?: boolean;
}
/**
 * Defines a function to resolve remote chunk's URL used in {@link ChunkManagerConfig}.
 * It's an async function which should return an object with `url` to a remote location
 * from where {@link ChunkManager} will download the chunk.
 */
export declare type RemoteChunkResolver = (chunkId: string, parentChunkId?: string) => Promise<RemoteChunkLocation>;
/**
 * Interface for storage backend used in {@link ChunkManagerConfig}.
 * The interface is modelled on Async Storage from `react-native-community`.
 */
export interface StorageApi {
    /** Gets the data for the key. */
    getItem: (key: string) => Promise<string | null | undefined>;
    /** Sets the item value based on the key. */
    setItem: (key: string, value: string) => Promise<void>;
    /** Removes the item based on the key. */
    removeItem: (key: string) => Promise<void>;
}
/**
 * Configuration options for {@link ChunkManager}.
 */
export interface ChunkManagerConfig {
    /**
     * Optional: A storage backend to cache resolved URLs for chunks.
     * The stored data is used to detect if URL to previously downloaded
     * chunk hasn't changed to avoid over-fetching the chunk.
     * If the perviously resolved URL matches new URL, the chunk won't be downloaded
     * again and the previously downloaded chunk will be executed instead.
     */
    storage?: StorageApi;
    /**
     * An async function to resolve URL to remote chunks hosted on remove servers.
     * You can use remote config, feature flags or A/B testing inside this function
     * return different URLs based on this logic.
     */
    resolveRemoteChunk: RemoteChunkResolver;
    /**
     * Forces `ChunkManager` to always use `resolveRemoteChunk` function to resolve location
     * of a chunk, regardless if the chunk is marked as local chunk or if the development server
     * is running.
     */
    forceRemoteChunkResolution?: boolean;
}
/**
 * A manager to ease resolving, downloading and executing additional code from async chunks or
 * any arbitrary JavaScript files.
 *
 * - In development mode, all chunks will be resolved and downloaded from the Development server.
 * - In production mode, local chunks will be resolved and loaded from filesystem and remote
 * chunks will be resolved and downloaded based on the `resolveRemoteChunk` function.
 * - You can force all resolution, regardless of the mode, to go through `resolveRemoteChunk`
 * function by setting `forceRemoteChunkResolution: true` in `ChunkManager.configure(...)`.
 *
 * This API is only useful if you are working with any form of Code Splitting.
 *
 * Example of using this API with async chunks:
 * ```js
 * import * as React from 'react';
 * import { ChunkManager } from '@callstack/repack/client';
 *
 * ChunkManager.configure({
 *   resolveRemoteChunk: async (chunkId) => {
 *     return {
 *       url: `http://domain.exaple/apps/${chunkId}`,
 *     };
 *   },
 * });
 *
 * // ChunkManager.loadChunk is called internally when running `import()`
 * const TeacherModule = React.lazy(() => import('./Teacher.js'));
 * const StudentModule = React.lazy(() => import('./Student.js'));
 *
 * export function App({ role }) {
 *   if (role === 'teacher') {
 *     return <TeacherModule />;
 *   }
 *
 *   return <StudentModule />
 * }
 * ```
 */
export declare class ChunkManager {
    /**
     * A instance of `ChunkManagerBackend`.
     * Should not be used directly.
     *
     * @internal
     */
    private static backend;
    /**
     * Configures `ChunkManager` to be able to resolve location of additional
     * chunks (or arbitrary code) in production.
     * Optionally, it also allows to set up caching to avoid over-fetching of chunks.
     *
     * @param config Configuration options.
     */
    static configure(config: ChunkManagerConfig): void;
    /**
     * Resolves a URL to a given chunks and  whether to download a chunk
     * or reuse previously downloaded one.
     *
     * @param chunkId Id of the chunk.
     * @returns Promise with chunk's URL as `url` and a boolean `fetch` whether to download a chunk
     * or reuse previously downloaded one.
     */
    static resolveChunk(chunkId: string): Promise<{
        url: string;
        fetch: boolean;
    }>;
    /**
     * Resolves given chunk's location, download and execute it.
     * Once the returned Promise is resolved, the code should have been evaluated.
     *
     * The execution of the code is handled internally by threading in React Native.
     *
     * @param chunkId Id of the chunk.
     * @param parentChunkId Id of the parent chunk.
     */
    static loadChunk(chunkId: string, parentChunkId?: string): Promise<void>;
    /**
     * Resolves given chunk's location and download it without executing.
     *
     * This function can be awaited to detect if the chunk was downloaded and for error handling.
     *
     * @param chunkId Id of the chunk.
     */
    static preloadChunk(chunkId: string): Promise<void>;
    /**
     * Clears the cache (if configured in {@link ChunkManager.configure}) and removes downloaded
     * files for given chunks from the filesystem.
     *
     * This function can be awaited to detect if the chunks were invalidated and for error handling.
     *
     * @param chunksIds Array of chunk Ids to clear from cache and remove from filesystem.
     */
    static invalidateChunks(chunksIds?: string[]): Promise<void>;
}
/**
 * A helper class to ease the creation of of chunk based on it's location.
 *
 * **You should not need to use this.**
 *
 * @internal
 */
export declare class Chunk {
    /**
     * Creates definition for a chunk hosted on development server.
     *
     * @param chunkId Id of the chunk.
     * @returns Chunk definition.
     */
    static fromDevServer(chunkId: string): string;
    /**
     * Creates definition for a chunk stored on filesystem on the target mobile device.
     *
     * @param chunkId Id of the chunk.
     * @returns Chunk definition.
     */
    static fromFileSystem(chunkId: string): string;
    /**
     * Creates definition for a chunk hosted on a remote server.
     *
     * @param url A URL to remote location where the chunk is stored.
     * @param options Additional options.
     * @returns Chunk definition.
     */
    static fromRemote(url: string, options?: {
        excludeExtension?: boolean;
    }): string;
}
